using namespace std;

#include <iostream>
#include <unordered_set>
#include <queue>
#include "testlib.h"

typedef long long ll;
typedef pair<ll, ll> point;

#define oxaxis point(0, 0), point(1, 0)
#define whilelim(cond) for(int i__ =  0; (cond) && i__ < 100; ++i__)

class DSU {
    vector<int> p, w;
public:
    DSU() = default;

    DSU(int count) : p(count + 1), w(count + 1, 0) {
        for (int i = 0; i < count; ++i) { p[i] = i; }
    }

    int find(int v) {
        while (p[v] != v) { v = p[v]; }
        return v;
    }

    void join(int a, int b) {
        a = find(a), b = find(b);
        if (a == b) return;
        if (w[a] == w[b]) w[a]++;
        if (w[a] < w[b]) p[a] = b;
        else p[b] = a;
    }
};

const double PI = acos(-1.0);
const double PId4 = atan(1.0);

vector<point> points;
map<point, int> used_point;
set<pair<int, int>> used_edges;
vector<pair<int, int >> edges;
DSU dsuTree;
bool gen_useless_edge, gen_tree;

ll dist(const point a, const point b) {
    return (a.first - b.first) * (a.first - b.first) + (a.second - b.second) * (a.second - b.second);
}

ll scvmp(const point a1, const point a2, const point b1, const point b2) {
    ll x1 = a2.first - a1.first, x2 = b2.first - b1.first, y1 = a2.second - a1.second, y2 = b2.second - b1.second;
    return x1 * x2 + y1 * y2;
}

ll vectmp(const point a1, const point a2, const point b1, const point b2) {
    ll x1 = a2.first - a1.first, x2 = b2.first - b1.first, y1 = a2.second - a1.second, y2 = b2.second - b1.second;
    return x1 * y2 - x2 * y1;
}

// angle (a1->a2)^(b1->b2)
double angle(const point a1, const point a2, const point b1, const point b2) {
    return atan2(static_cast<double>(vectmp(a1, a2, b1, b2)), static_cast<double>(scvmp(a1, a2, b1, b2)));
}


int add_point(map<point, int> &used_point, ll x, ll y) {
    if (used_point.find(make_pair(x, y)) != used_point.end()) {
        return used_point[make_pair(x, y)];
    }
    int pb = points.size();
    points.emplace_back(x, y);
    used_point[make_pair(x, y)] = pb;
    return pb;
}

int gen_point(ll max_coord) {
    ll x = rnd.next(1LL, max_coord - 1);
    ll y = rnd.next(1LL, max_coord - 1);
    return add_point(used_point, x, y);
}

bool is_correct_seg(int ps, int p, int u, int r, int l) {
    double lang = angle(points[u], points[p], points[l], points[p]);
    double rang = angle(points[u], points[p], points[r], points[p]);
    return (0 == p || abs(angle(points[p], points[0], points[u], points[p])) < PId4)
           && abs(lang) < PId4 && abs(rang) < PId4 && (lang > 0 && rang < 0 || lang < 0 && rang > 0)
           && (p == 0 || abs(angle(points[p], points[ps], points[u], points[p])) < PId4)
           && dist(points[l], points[p]) < dist(points[u], points[p]) &&
           dist(points[r], points[p]) < dist(points[u], points[p]);
}

bool is_in_sector(const point from, double ang_b, double ang_e, double r, ll x, ll y) {
    double ang = angle(oxaxis, from, point(x, y));
    return ang_b < ang && ang < ang_e && dist(from, point(x, y)) < r;
}

int gen_point(ll max_coord, point from,
              double ang_b, double ang_e, double r) {
    ll x = -1, y = -1;
    r = max(2.0, min(r, (double) min(min(abs(max_coord - from.first), abs(-max_coord - from.first)),
                                     min(abs(max_coord - from.second), abs(-max_coord - from.second)))));
//    cout << "gen p " << r << " " << ang_b << " " << ang_e << "\n";
    whilelim (x == -1 || !is_in_sector(from, ang_b, ang_e, r, x, y)) {
//        if (i__ == 50) { cout << "ohh\n"; }
        if (ang_b >= ang_e) { swap(ang_b, ang_e); }
        double new_ang = rnd.next(ang_b, ang_e);
        double new_r = rnd.next(1.0, r);
        double dx = new_r * cos(new_ang), dy = new_r * sin(new_ang);
        x = max(1.0, from.first + dx);
        y = max(1.0, from.second + dy);
//        cout << "? " << x << " " << y << " \n";
        if (is_in_sector(from, ang_b, ang_e, r, x + 1, y)) { ++x; }
        else if (is_in_sector(from, ang_b, ang_e, r, x, y + 1)) { ++y; }
        else if (is_in_sector(from, ang_b, ang_e, r, x + 1, y + 1)) {
            ++x;
            ++y;
        }
    }
    return add_point(used_point, x, y);
}

bool add_edge(int x, int y) {
    if (x == y) { return false; }
    if (used_edges.find(make_pair(x, y)) != used_edges.end() || used_edges.find(make_pair(y, x)) != used_edges.end()
        || gen_tree && dsuTree.find(x) == dsuTree.find(y)) {
        return false;
    }
    edges.emplace_back(x, y);
    used_edges.insert(make_pair(x, y));
    used_edges.insert(make_pair(y, x));
    dsuTree.join(x, y);
    return true;
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    int max_n = opt<int>("n");
    int max_m = opt<int>("m");
    ll max_coord = opt<ll>("c");
    ll med_edge = opt<ll>("e");
    int trunk_prev = opt<int>("p");
    gen_tree = opt<bool>("gt");
    gen_useless_edge = opt<bool>("ue");
    bool gen_complete_graph = opt<bool>("cg");

    if (gen_complete_graph) {
        max_n = min(max_n, static_cast<int>(sqrt(max_m * 2)));
        max_m = (int) min((ll) max_m, ((ll) max_n) * ((ll) max_n - 1) / 2);
    }

    int n = 4 + rnd.wnext(max_n - 3, 2);
    int m = 4 + rnd.wnext(max_m - 3, 2);
    if (gen_complete_graph) {
//        gen_useless_edge = true;
        m = n * (n - 1) / 2;
    }
    //    int seg = rnd.next(1, (m - 1) / 3);
    int seg = (n - 1) / 3;

    used_point = map<point, int>();
    used_edges = set < pair < int, int >> ();
    edges = vector < pair < int, int >> ();
    points = vector<point>();
    dsuTree = DSU(n);
    vector<int> trunk = {0};
    map<int, int> trunkp;
    trunkp[0] = -1;

    if (gen_point(max_coord) != 0) { return 0; }
    for (int i = 0; i < seg; ++i) {
        int ctrunk_prev = rnd.next(1, min(trunk_prev, static_cast<int>(trunk.size())));
        int p = trunk[trunk.empty() ? 0 :
                      static_cast<int>(trunk.size()) - ctrunk_prev + rnd.wnext(ctrunk_prev, 3)];
        int parpn = trunkp[p];
        int u = p;
        if (parpn == -1) {
            whilelim(u == p) {
                u = gen_point(max_coord, points[p], -PI, PI, med_edge);
            }
        } else {
            double ang1 = angle(oxaxis, points[parpn], points[p]);
            double ang2 = angle(oxaxis, points[0], points[p]);
            double ang1l = ang1 - PId4;
            double ang2l = ang2 - PId4;
            double ang1r = ang1 + PId4;
            double ang2r = ang2 + PId4;
            double angl = abs(ang2l - ang1l) > PI ?
                          max(max(ang1l, ang2l) - 2 * PI, min(ang1l, ang2l)) : max(ang1l, ang2l);
            double angr = abs(ang2r - ang1r) > PI ?
                          min(max(ang1r, ang2r) - 2 * PI, min(ang1r, ang2r)) : min(ang1r, ang2r);
            if (angr < angl) swap(angr, angl);
            whilelim(u == p) {
                u = gen_point(max_coord, points[p], angl, angr, med_edge);
            }
        }
        double ang = angle(oxaxis, points[p], points[u]);

        int l = u, r = u;
        whilelim (l == u || l == p) {
            l = gen_point(max_coord, points[p], ang - PId4, ang,
                          sqrt(dist(points[p], points[u])));
        }
        whilelim (r == u || r == p || r == l) {
            r = gen_point(max_coord, points[p], ang, ang + PId4,
                          sqrt(dist(points[p], points[u])));
        }

        add_edge(p, u);
        if (p != u && is_correct_seg(parpn, p, u, l, r)) {
            trunk.push_back(u);
            trunkp[u] = p;
        }
        add_edge(p, r);
        add_edge(p, l);

    }
    int infLoopCounter = 0;
    while (infLoopCounter < 1000 && points.size() < n && edges.size() + 1 <= max_m) {
        int points_count = points.size();
        int u = gen_point(max_coord);
        if (u < points_count) {
            ++infLoopCounter;
        } else {
            infLoopCounter = 0;
            add_edge(u, rnd.next(0, points_count - 1));
        }
    }
    vector <unordered_set<int>> incident_edges(n);
    for (int i = 0; i < n; i++) {
        incident_edges[i].insert(i);
    }
    for (auto eg : edges) {
        incident_edges[eg.first].insert(eg.second);
        incident_edges[eg.second].insert(eg.first);
    }
    while (gen_useless_edge && edges.size() < m && edges.size() < ((ll) n - 1) * (ll) n / 2) {
        int u = (int) rnd.next(points.size());
        while (incident_edges[u].size() == n) u = (u + 1) % n;
        int v = rnd.next(n);
        while (incident_edges[u].count(v)) v = (v + 1) % n;
        incident_edges[u].insert(v);
        incident_edges[v].insert(u);
        add_edge(u, v);
    }

    vector<int> pperm = rnd.perm(n), piperm(n);
    for (int i = 0; i < n; ++i) { piperm[pperm[i]] = i; }
    printf("%d %d %d\n", (int) points.size(), (int) edges.size(), pperm[0] + 1);
    for (int i = 0; i < points.size(); ++i) {
        printf("%lld %lld\n", points[piperm[i]].first, points[piperm[i]].second);
    }
    printf("\n");
    shuffle(edges.begin(), edges.end());
    for (int i = 0; i < edges.size(); ++i) {
        printf("%d %d\n", pperm[edges[i].first] + 1, pperm[edges[i].second] + 1);
    }
}

