using namespace std;

#include <iostream>
#include <unordered_set>
#include <queue>
#include "testlib.h"

typedef long long ll;
typedef pair<ll, ll> point;

#define oxaxis point(0, 0), point(1, 0)
#define whilelim(cond) for(int i__ =  0; (cond) && i__ < 100; ++i__)

const double PI = acos(-1.0);
const double PId4 = atan(1.0);

vector<point> points;
map<point, int> used_point;
set<pair<int, int>> used_edges;
vector<pair<int, int >> edges;
bool gen_useless_edge;
ll med_edge, max_coord;

ll dist(const point a, const point b) {
    return (a.first - b.first) * (a.first - b.first) + (a.second - b.second) * (a.second - b.second);
}

ll scvmp(const point a1, const point a2, const point b1, const point b2) {
    ll x1 = a2.first - a1.first, x2 = b2.first - b1.first, y1 = a2.second - a1.second, y2 = b2.second - b1.second;
    return x1 * x2 + y1 * y2;
}

ll vectmp(const point a1, const point a2, const point b1, const point b2) {
    ll x1 = a2.first - a1.first, x2 = b2.first - b1.first, y1 = a2.second - a1.second, y2 = b2.second - b1.second;
    return x1 * y2 - x2 * y1;
}

// angle (a1->a2)^(b1->b2)
double angle(const point a1, const point a2, const point b1, const point b2) {
    return atan2(static_cast<double>(vectmp(a1, a2, b1, b2)), static_cast<double>(scvmp(a1, a2, b1, b2)));
}


int add_point(ll x, ll y) {
    if (used_point.find(make_pair(x, y)) != used_point.end()) {
        return used_point[make_pair(x, y)];
    }
    int pb = points.size();
    points.emplace_back(x, y);
    used_point[make_pair(x, y)] = pb;
    return pb;
}

bool add_edge(int x, int y) {
    if (x == y) { return false; }
    if (used_edges.find(make_pair(x, y)) != used_edges.end() || used_edges.find(make_pair(y, x)) != used_edges.end()) {
        return false;
    }
    edges.emplace_back(x, y);
    used_edges.insert(make_pair(x, y));
    used_edges.insert(make_pair(y, x));
    return true;
}

int gen_point() {
    ll x = rnd.next(0, 1);
    ll y = rnd.next(1LL, max_coord - 1);
    return add_point(x, y);
}

bool is_valid_segment(point pp, point p, point u, point l, point r) {
    double up_ps = angle(p, points[0], u, p);
    double up_ppp = angle(u, p, p, pp);
    double up_lp = angle(u, p, l, p);
    double up_rp = angle(u, p, r, p);
    bool c1 = (p == points[0] || -PId4 < up_ps && up_ps < PId4 && -PId4 < up_ppp && up_ppp < PId4);
    bool c2 = -PId4 < up_rp && up_rp < 0 && 0 < up_lp && up_lp < PId4;
    bool c3 = dist(l, p) < dist(u, p) && dist(r, p) < dist(u, p);
    return c1
           && c2
           && c3;
}

int gen_seg_point(int from, point pp, int sign) {
    point p = points[from];
    ll maxdy = sign > 0 ? max_coord - p.second : p.second;
    maxdy = min(maxdy, med_edge);
    point u, l, r;
    u.second = -1;
    u.first = 1 - p.first;
    l.first = sign > 0 ? 0 : 1;
    r.first = sign > 0 ? 1 : 0;
//    cout << "gen p " << r << " " << ang_b << " " << ang_e << "\n";
    whilelim (u.second == -1 || !is_valid_segment(pp, p, u, l, r)) {
        ll dy = rnd.next(1LL, maxdy);
        u.second = max(1LL, p.second + dy * sign);
        l.second = max(1LL, p.second + rnd.next(1LL, dy) * sign);
        r.second = max(1LL, p.second + rnd.next(1LL, dy) * sign);

    }
    int ln = add_point(l.first, l.second);
    int rn = add_point(r.first, r.second);
    add_edge(from, ln);
    add_edge(from, rn);
    int un = add_point(u.first, u.second);
    add_edge(from, un);
//    cout << "ok ? " << is_valid_segment(s, pp, p, u, l, r) << " " << un << "\n";
    return is_valid_segment(pp, p, u, l, r) ? un : -1;
}


int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    int max_n = opt<int>("n");
    int max_m = opt<int>("m");
    max_coord = opt<ll>("c");
    med_edge = opt<ll>("e");
    int trunk_prev = opt<int>("p");
    gen_useless_edge = opt<bool>("ue");

    int n = 4 + rnd.wnext(max_n - 3, 2);
    int m = 4 + rnd.wnext(max_m - 3, 2);
    //    int seg = rnd.next(1, (m - 1) / 3);
    int seg = (n - 1) / 3;

    used_point = map<point, int>();
    used_edges = set < pair < int, int >> ();
    edges = vector < pair < int, int >> ();
    points = vector<point>();
    vector<int> trunk = {0};
    map<int, pair<int, int>> trunkp;
    trunkp[0] = {-1, 0};

    if (gen_point() != 0) { return 0; }
    for (int i = 0; i < seg; ++i) {
        int ctrunk_prev = rnd.next(1, min(trunk_prev, static_cast<int>(trunk.size())));
        int p = trunk[trunk.empty() ? 0 :
                      static_cast<int>(trunk.size()) - ctrunk_prev + rnd.wnext(ctrunk_prev, 3)];
        int parpn = trunkp[p].first;
        int sign = trunkp[p].second;
        int u;
        if (parpn == -1) {
            sign = rnd.next(0, 1) ? 1 : -1;
            u = gen_seg_point(p, points[p], sign);
        } else {
            u = gen_seg_point(p, points[parpn], sign);
        }

        if (u > -1) {
//            cout << " " << u << " " << p << " " << sign << "\n";
            trunk.push_back(u);
            trunkp[u] = {p, sign};
        }
    }
    int infLoopCounter = 0;
    while (infLoopCounter < 1000 && points.size() < n) {
        int points_count = points.size();
        int u = gen_point();
        if (u < points_count) {
            ++infLoopCounter;
        } else if (edges.size() < m) {
            add_edge(u, rnd.next(0, points_count - 1));
        }
    }

    vector <unordered_set<int>> incident_edges(n);
    for (int i = 0; i < n; i++) {
        incident_edges[i].insert(i);
    }
    for (auto eg : edges) {
        incident_edges[eg.first].insert(eg.second);
        incident_edges[eg.second].insert(eg.first);
    }
    infLoopCounter = 0;
    while (gen_useless_edge && infLoopCounter < 10000
           && edges.size() < m && edges.size() < ((ll) n - 1) * (ll) n / 2) {
        int u = (int)rnd.next(points.size());
        while (incident_edges[u].size() == n) u = (u + 1) % n;
        int v = rnd.next(n);
        while (incident_edges[u].count(v)) v = (v + 1) % n;
        incident_edges[u].insert(v);
        incident_edges[v].insert(u);
        infLoopCounter++;
        if (add_edge(u, v)) {
            infLoopCounter = 0;
        }
    }

//    vector<int> pperm(n), piperm(n);
    vector<int> pperm = rnd.perm(n), piperm(n);
    for (int i = 0; i < n; ++i) { pperm[i] = i; }
    for (int i = 0; i < n; ++i) { piperm[pperm[i]] = i; }
    printf("%d %d %d\n", (int) points.size(), (int) edges.size(), pperm[0] + 1);
    for (int i = 0; i < points.size(); ++i) {
        printf("%lld %lld\n", points[piperm[i]].first, points[piperm[i]].second);
    }
    printf("\n");
    shuffle(edges.begin(), edges.end());
    for (int i = 0; i < edges.size(); ++i) {
        printf("%d %d\n", pperm[edges[i].first] + 1, pperm[edges[i].second] + 1);
    }
}

