#include <functional>
#include <algorithm>
#include "testlib.h"

using namespace std;

int main(int argc, char *argv[]) {

    registerValidation(argc, argv);
    int group = validator.group().empty() ? -1 : atoi(validator.group().c_str());

    int n = inf.readInt(2, 200000, "n");
    inf.readSpace();
    int m = inf.readInt(1, 400000, "m");
    inf.readSpace();
    inf.readInt(1, n, "s") - 1;
    inf.readEoln();

    set<pair<int, int>> points_coords;
    for (int i = 0; i < n; i++) {
        int x = inf.readInt(0, 1000'000'000, format("x_%d", i + 1));
        if (group == 4) ensuref(x <= 1, "x_%d > 1", i + 1);
        inf.readSpace();
        int y = inf.readInt(0, 1000'000'000, format("y_%d", i + 1));
        inf.readEoln();

        ensuref(points_coords.find(make_pair(x, y)) == points_coords.end(), "more than one node have same coordinates");
        points_coords.emplace(x, y);
    }
    inf.readEoln();

    vector<int> p(n), r(n, 0);
    for (int i = 0; i < n; i++) p[i] = i;
    function<int(int)> get = [&](int t) {
        if (p[t] == t) return t;
        return p[t] = get(p[t]);
    };
    function<void(int, int)> unite = [&](int a, int b) {
        a = get(a);
        b = get(b);
        if (r[a] > r[b]) swap(a, b);
        p[a] = b;
        if (r[a] == r[b]) r[b]++;
    };

    set<pair<int, int>> edges;
    for (int i = 0; i < m; i++) {
        int u = inf.readInt(1, n, format("u_%d", i + 1));
        inf.readSpace();
        int v = inf.readInt(1, n, format("u_%d", i + 1));
        ensuref(u != v, "u_%d = v_%d", i + 1, i + 1);
        unite(u - 1, v - 1);
        ensuref(edges.find(make_pair(u, v)) == edges.end(), "should be no multiple edges");
        edges.emplace(u, v);
        edges.emplace(v, u);
        inf.readEoln();
    }

    if (group == 1) {
        ensuref(n <= 5, "n = %d is greater than 5", n);
    } else if (group == 2) {
        ensuref(n <= 10, "n = %d is greater than 10", n);
    } else if (group == 3) {
        for (int i = 1; i < n; i++)
            ensuref(get(i) == get(0), "vertices 1 and %d are unreachable", i + 1);
        ensuref(m == n - 1, "count of edges not equal n - 1, it isn't tree");
    } else if (group == 5) {
        ensuref(n <= 1000, "n = %d is greater than 1000", n);
        ensuref(m <= 10000, "m = %d is greater than 10000", m);
    }

    inf.readEof();

}