//  Copyright 2022 Nikita Golikov

#include <bits/stdc++.h>

using namespace std;

using uint = unsigned int;
using ll = long long;
using ull = unsigned long long;

#ifdef GOLIKOV
#define debug(x) cerr << (#x) << ":\t" << (x) << endl
#else
#define debug(x) 238;
#endif

template <class A, class B>
bool smin(A &x, B &&y) {
    if (y < x) {
        x = y;
        return true;
    }
    return false;
}

template <class A, class B>
bool smax(A &x, B &&y) {
    if (x < y) {
        x = y;
        return true;
    }
    return false;
}

template <class T>
struct Point {
    T x{}, y{};

    Point() = default;

    Point(T x_, T y_) : x(x_), y(y_) {}

    Point operator+(const Point& other) const {
        return Point(x + other.x, y + other.y);
    }

    Point operator-(const Point& other) const {
        return Point(x - other.x, y - other.y);
    }

    Point operator+() const {
        return *this;
    }

    Point operator-() const {
        return Point(-x, -y);
    }

    Point operator*(T k) const {
        return Point(x * k, y * k);
    }

    Point operator/(T k) const {
        return Point(x / k, y / k);
    }

    T operator*(const Point& other) const {
        return x * other.x + y * other.y;
    }

    T operator%(const Point& other) const {
        return x * other.y - y * other.x;
    }

    T lenSqr() const {
        return x * x + y * y;
    }

    double len() const {
        return sqrt(lenSqr());
    }

    T distSqr(const Point& other) const {
        return (other - *this).lenSqr();
    }

    double dist(const Point& other) const {
        return sqrt(distSqr(other));
    }

    Point norm() const {
        return *this / len();
    }

    // Counterclockwise rotation by 90 degrees
    Point ort() const {
        return Point(-y, x);
    }

    // Counterclockwise rotation by c = cos(alpha), s = sin(alpha)
    Point rot(double c, double s) const {
        return *this * c + ort() * s;
    }

    // Counterclockwise rotation by alpha
    Point rot(double alpha) const {
        return rot(cos(alpha), sin(alpha));
    }

    double arg() const {
        return atan2(y, x);
    }

    long double argl() const {
        return atan2l(y, x);
    }

    friend istream& operator>>(istream& in, Point& p) {
        return in >> p.x >> p.y;
    }

    friend ostream& operator<<(ostream& out, Point p) {
        return out << p.x << ' ' << p.y;
    }

    bool up() const {
        return y > 0 || (y == 0 && x >= 0);
    }

    bool operator==(Point const& other) const {
        return x == other.x && y == other.y;
    }

    bool operator!=(Point const& other) const {
        return x != other.x || y != other.y;
    }

    friend auto angleCmp(Point O) {
        return [O](Point x, Point y) {
            x = x - O;
            y = y - O;
            auto d = x.up() - y.up();
            if (d == 0) {
                auto vmul = x % y;
                return vmul == 0 ? x.lenSqr() < y.lenSqr() : vmul > 0;
            }
            return d < 0;
        };
    }

    friend auto angleCmp() {
        return angleCmp(Point());
    }

    bool operator<(Point const& other) const {
        return tie(x, y) < tie(other.x, other.y);
    }

    friend bool ccw(Point A, Point B, Point C) {
        return (B - A) % (C - A) >= 0;
    }

    //  polygon ccw
    friend bool insidePolygon(vector<Point> const& p, Point O) {
        for (int i = 0; i < int(p.size()); ++i) {
            if (!ccw(O, p[i], p[(i + 1) % int(p.size())])) return false;
        }
        return true;
    }
};

int main() {
#ifdef GOLIKOV
    assert(freopen("in", "rt", stdin));
  auto _clock_start = chrono::high_resolution_clock::now();
#endif
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, m, s;
    cin >> n >> m >> s;
    --s;
    using P = Point<ll>;
    vector<P> a(n);
    for (auto& p : a) {
        cin >> p;
    }
    using ld = long double;
    ld const PI = acosl(-1.0);
    ld const EPS = 1e-9;

    auto ang2 = [&](P x, P y) {
        return atan2(x % y, x * y);
    };
    auto angle = [&](int i, int j, int k) {
        return ang2(a[k] - a[i], a[j] - a[i]);
    };
    auto le45 = [&](int i, int j, int k) {
        return fabs(angle(i, j, k)) < PI / 4 - EPS;
    };
    auto left45 = [&](int i, int j, int k) {
        auto ang = ang2(a[j] - a[i], a[k] - a[i]);
        return ang > EPS && ang < PI / 4 - EPS;
    };
    auto right45 = [&](int i, int j, int k) {
        auto ang = ang2(a[j] - a[i], a[k] - a[i]);
        return ang < -EPS && ang > -PI / 4 + EPS;
    };

    vector<vector<pair<int, ll>>> graph(n);
    for (int i = 0; i < m; ++i) {
        int u, v;
        cin >> u >> v;
        --u, --v;
        ll len = a[u].distSqr(a[v]);
        if (u == s) {
            graph[u].emplace_back(v, len);
            continue;
        }
        if (v == s) {
            graph[v].emplace_back(u, len);
            continue;
        }
        if (le45(s, u, v)) {
            graph[u].emplace_back(v, len);
        }
        if (le45(s, v, u)) {
            graph[v].emplace_back(u, len);
        }
    }
    struct edge {
        ll len;
        int to;
        int L;
        int R;
    };
    vector<vector<edge>> g(n);
    vector<vector<int>> ng(n);
    for (int i = 0; i < n; ++i) {
        for (auto[j, l] : graph[i]) {
            int L = -1;
            int R = -1;
            for (auto[k, lk] : graph[i]) {
                if (lk > l) {
                    continue;
                }
                if (left45(i, j, k)) {
                    L = k;
                }
                if (right45(i, j, k)) {
                    R = k;
                }
            }
            if (L >= 0 && R >= 0) {
                g[i].push_back({l, j, L, R});
                ng[i].push_back(j);
            }
            // int L = -1;
            // int R = -1;
            // for (auto[k, lk] : graph[i]) {
            //   if (L >= 0) {
            //     break;
            //   }
            //   if (lk > l || !left45(i, j, k)) {
            //     continue;
            //   }

            //   for (auto[t, lt] : graph[i]) {
            //     if (lt > l || !right45(i, j, t)) {
            //       continue;
            //     }
            //     L = k;
            //     R = t;
            //     break;
            //   }
            // }
            // if (L >= 0) {
            // g[i].push_back({l, j, L, R});
            // ng[i].push_back(j);
            // }
        }
    }
    vector<int> deg(n);
    for (int i = 0; i < n; ++i) {
        for (int j : ng[i]) {
            deg[j]++;
        }
    }
    vector<int> ts;
    for (int i = 0; i < n; ++i) {
        if (deg[i] == 0) {
            ts.push_back(i);
        }
    }
    for (int i = 0; i < int(ts.size()); ++i) {
        int v = ts[i];
        for (int u : ng[v]) {
            if (--deg[u] == 0) {
                ts.push_back(u);
            }
        }
    }

    auto okPair = [&](int i, int j, int k) {
        return le45(i, j, k);
    };

    vector<vector<ll>> dp(n);
    vector<vector<int>> parent(n);
    for (int i = n - 1; i >= 0; --i) {
        int v = ts[i];
        for (int j = 0; j < int(g[v].size()); ++j) {
            auto[len, to, L, R] = g[v][j];
            dp[v].push_back(len);
            parent[v].push_back(-1);
            for (int k = 0; k < int(g[to].size()); ++k) {
                int to2 = g[to][k].to;
                if (okPair(v, to, to2) && smax(dp[v][j], len + dp[to][k])) {
                    parent[v][j] = k;
                }
            }
        }
    }
    if (dp[s].empty()) {
        cout << 0 << ' ' << 0 << '\n';
        return 0;
    }
    int v = s;
    int e = max_element(dp[s].begin(), dp[s].end()) - dp[s].begin();
    ll ans = dp[v][e];
    vector<tuple<int, int, int>> path;
    while (true) {
        path.emplace_back(g[v][e].L, g[v][e].to, g[v][e].R);
        int ne = parent[v][e];
        if (ne < 0) {
            break;
        }
        v = g[v][e].to;
        e = ne;
    }
    cout << path.size() << ' ' << ans << '\n';
    for (auto[L, x, R] : path) {
        cout << L + 1 << ' ' << x + 1 << ' ' << R + 1 << '\n';
    }

#ifdef GOLIKOV
    cerr << "Executed in " << chrono::duration_cast<chrono::milliseconds>(
      chrono::high_resolution_clock::now()
          - _clock_start).count() << "ms." << endl;
#endif
    return 0;
}