//  Copyright 2022 Nikita Golikov
 
#include <bits/stdc++.h>
 
using namespace std;
 
using uint = unsigned int;
using ll = long long;
using ull = unsigned long long;
 
#ifdef GOLIKOV
    #define debug(x) cerr << (#x) << ":\t" << (x) << endl
#else
    #define debug(x) 238;
#endif
 
template <class A, class B>
bool smin(A &x, B &&y) {
  if (y < x) {
    x = y;
    return true;
  }
  return false;
}
 
template <class A, class B>
bool smax(A &x, B &&y) {
  if (x < y) {
    x = y;
    return true;
  }
  return false;
}
 
template <class T>
struct Point {
    T x{}, y{};
 
    Point() = default;
 
    Point(T x_, T y_) : x(x_), y(y_) {}
 
    Point operator+(const Point& other) const {
        return Point(x + other.x, y + other.y);
    }
 
    Point operator-(const Point& other) const {
        return Point(x - other.x, y - other.y);
    }
 
    Point operator+() const {
        return *this;
    }
 
    Point operator-() const {
        return Point(-x, -y);
    }
 
    Point operator*(T k) const {
        return Point(x * k, y * k);
    }
 
    Point operator/(T k) const {
        return Point(x / k, y / k);
    }
 
    T operator*(const Point& other) const {
        return x * other.x + y * other.y;
    }
 
    T operator%(const Point& other) const {
        return x * other.y - y * other.x;
    }
 
    T lenSqr() const {
        return x * x + y * y;
    }
 
    double len() const {
        return sqrt(lenSqr());
    }
 
    T distSqr(const Point& other) const {
        return (other - *this).lenSqr();
    }
 
    double dist(const Point& other) const {
        return sqrt(distSqr(other));
    }
 
    Point norm() const {
        return *this / len();
    }
 
    // Counterclockwise rotation by 90 degrees
    Point ort() const {
        return Point(-y, x);
    }
 
    // Counterclockwise rotation by c = cos(alpha), s = sin(alpha)
    Point rot(double c, double s) const {
        return *this * c + ort() * s;
    }
 
    // Counterclockwise rotation by alpha
    Point rot(double alpha) const {
        return rot(cos(alpha), sin(alpha));
    }
 
    double arg() const {
        return atan2(y, x);
    }
 
    long double argl() const {
      return atan2l(y, x);
    }
 
    friend istream& operator>>(istream& in, Point& p) {
      return in >> p.x >> p.y;
    }
 
    friend ostream& operator<<(ostream& out, Point p) {
      return out << p.x << ' ' << p.y;
    }
 
    bool up() const {
        return y > 0 || (y == 0 && x >= 0);
    }
 
    bool operator==(Point const& other) const {
        return x == other.x && y == other.y;    
    }
 
    bool operator!=(Point const& other) const {
        return x != other.x || y != other.y;    
    }
 
    static auto angleCmp(Point O) {
      return [O](Point x, Point y) {
        x = x - O;
        y = y - O;
        auto d = x.up() - y.up();
        if (d == 0) {
          auto vmul = x % y;
          return vmul == 0 ? x.lenSqr() < y.lenSqr() : vmul > 0;
        }
        return d < 0;
      };
    }
 
    static auto angleCmp() {
      return angleCmp(Point());
    }
 
    bool operator<(Point const& other) const {
        return tie(x, y) < tie(other.x, other.y);
    }
 
    friend bool ccw(Point A, Point B, Point C) {
        return (B - A) % (C - A) >= 0;
    }
 
    //  polygon ccw
    friend bool insidePolygon(vector<Point> const& p, Point O) {
      for (int i = 0; i < int(p.size()); ++i) {
        if (!ccw(O, p[i], p[(i + 1) % int(p.size())])) return false;
      }
      return true;
    }
};
 
int main() {
#ifdef GOLIKOV
  assert(freopen("in", "rt", stdin));
  auto _clock_start = chrono::high_resolution_clock::now();
#endif
  ios::sync_with_stdio(false);
  cin.tie(nullptr);
 
  int n, m, s;
  cin >> n >> m >> s;
  --s;
  using P = Point<ll>;
  vector<P> a(n);
  for (auto& p : a) {
    cin >> p;
  }
  using ld = long double;
  ld const PI = acosl(-1.0);
  ld const EPS = 1e-9;
 
  auto ang2 = [&](P x, P y) {
    return atan2(x % y, x * y);
  };
  auto angle = [&](int i, int j, int k) {
    return ang2(a[j] - a[i], a[k] - a[j]);
  };
  auto le45 = [&](int i, int j, int k) {
    return fabs(angle(i, j, k)) < PI / 4 - EPS;
  };
 
  vector<vector<pair<int, ll>>> graph(n);
  for (int i = 0; i < m; ++i) {
    int u, v;
    cin >> u >> v;
    --u, --v;
    ll len = a[u].distSqr(a[v]);
    graph[u].emplace_back(v, -len);
    graph[v].emplace_back(u, -len);
    if (u == s) {
      graph[u].emplace_back(v, len);
      continue;
    }
    if (v == s) {
      graph[v].emplace_back(u, len);
      continue;
    }
    if (le45(s, u, v)) {
      graph[u].emplace_back(v, len);
    }
    if (le45(s, v, u)) {
      graph[v].emplace_back(u, len);
    }
  }
 
  struct edge {
    ll len;
    int to;
    int L;
    int R;
  };
  vector<vector<edge>> g(n);
  vector<vector<int>> ng(n);
  for (int i = 0; i < n; ++i) {
    vector<tuple<ld, int, ll>> edges;
    int sz = int(graph[i].size());
    vector<int> L(sz, -1), R(sz, -1);
    for (int j = 0; j < sz; ++j) {
      int to = graph[i][j].first;
      ll len = graph[i][j].second;
      auto angl = (a[to] - a[i]).arg();
      if (angl < 0) {
        angl += 2 * PI;
      }
      edges.emplace_back(angl, j, llabs(len));
      edges.emplace_back(angl + 2 * PI, j, llabs(len));
    }
    sort(edges.begin(), edges.end());
 
    auto work = [&](int iter) {
      vector<tuple<ld, int, ll>> st;
      int pst = 0;
      set<pair<ll, int>> setik;
      for (int j = 0; j < 2 * sz; ) {
        int t = j;
        while (t + 1 < 2 * sz && fabs(get<0>(edges[t + 1]) - get<0>(edges[t])) < EPS) {
          ++t;
        }
        while (pst < int(st.size()) && fabs(get<0>(st[pst]) - get<0>(edges[t])) > PI / 4 - EPS) {
          setik.erase(make_pair(get<2>(st[pst]), get<1>(st[pst])));
          ++pst;
        }
        if (!setik.empty()) {
          auto[len, index] = *setik.begin();
          for (int k = j; k <= t; ++k) {
            if (get<2>(edges[k]) >= len) {
              (iter ? L : R)[get<1>(edges[k])] = get<1>(edges[index]);
            }
          }
        }
        for (int k = j; k <= t; ++k) {
          st.emplace_back(get<0>(edges[k]), k, get<2>(edges[k]));
          setik.emplace(get<2>(edges[k]), k);
        }
 
        j = t + 1;
      }
    };
 
    work(0);
    reverse(edges.begin(), edges.end());
    work(1);
    for (int j = 0; j < sz; ++j) {
      if (L[j] >= 0 && R[j] >= 0) {
        auto e = graph[i][j];
        if (e.second >= 0) {
          L[j] = graph[i][L[j]].first;
          R[j] = graph[i][R[j]].first;
          g[i].push_back({e.second, e.first, L[j], R[j]});
          ng[i].push_back(e.first);
        }
      }
    }
  }
  vector<int> deg(n);
  for (int i = 0; i < n; ++i) {
    for (int j : ng[i]) {
      deg[j]++;
    }
  }
  vector<int> ts;
  for (int i = 0; i < n; ++i) {
    if (deg[i] == 0) {
      ts.push_back(i);
    }
  }
  for (int i = 0; i < int(ts.size()); ++i) {
    int v = ts[i];
    for (int u : ng[v]) {
      if (--deg[u] == 0) {
        ts.push_back(u);
      }
    }
  }
 
  using I = ull;
  vector<vector<I>> dp(n);
  vector<vector<int>> parent(n);
  vector<vector<tuple<P, int, int>>> qs(n);
 
  for (int i = 0; i < n; ++i) {
    for (int j = 0; j < int(g[i].size()); ++j) {
      auto[len, to, L, R] = g[i][j];
      qs[to].emplace_back(a[to] - a[i], i, j);
      dp[i].push_back(0);
      parent[i].push_back(-1);
    }
  }
 
  for (int it = n - 1; it >= 0; --it) {
    int v = ts[it];
    vector<tuple<ld, ll, int, int>> events;
    for (int j = 0; j < int(g[v].size()); ++j) {
      auto[len, to, L, R] = g[v][j];
      dp[v][j] += len;
      auto angl = (a[to] - a[v]).arg();
      if (angl < 0) {
        angl += 2 * PI;
      }
      events.emplace_back(angl, dp[v][j], v, j);
      events.emplace_back(angl + 2 * PI, dp[v][j], v, j);
    }
    for (auto[p, from, ei] : qs[v]) {
      auto angl = p.arg();
      if (angl < 0) {
        angl += 2 * PI;
      }
      events.emplace_back(angl, -1, from, ei);
      events.emplace_back(angl + 2 * PI, -1, from, ei);
    }
    sort(events.begin(), events.end());
 
    int ne = int(events.size());
    auto work = [&] {
      set<pair<I, int>> vals;
      int j = 0;
      for (int i = 0; i < ne; ++i) {
        while (j < i && fabs(get<0>(events[i]) - get<0>(events[j])) > PI / 4 - EPS) {
          if (get<1>(events[j]) >= 0) {
            vals.erase(make_pair(get<1>(events[j]), get<3>(events[j])));
          }
          ++j;
        }
        if (get<1>(events[i]) >= 0) {
          vals.emplace(get<1>(events[i]), get<3>(events[i]));
        } else if (!vals.empty()) {
          auto[val, who] = *vals.rbegin();
          if (smax(dp[get<2>(events[i])][get<3>(events[i])], val)) {
            parent[get<2>(events[i])][get<3>(events[i])] = who;
          }
        }
      }
    };
 
    work();
    reverse(events.begin(), events.end());
    work();
  }
  if (dp[s].empty()) {
    cout << 0 << ' ' << 0 << '\n';
    return 0;
  }
  int v = s;
  int e = max_element(dp[s].begin(), dp[s].end()) - dp[s].begin();
  I ans = dp[v][e];
  assert(ans >= 0 && ans <= (ll) 9e18);
  vector<tuple<int, int, int>> path;
  I actAns = 0;
  while (true) {
    actAns += a[g[v][e].to].distSqr(a[v]);
    path.emplace_back(g[v][e].L, g[v][e].to, g[v][e].R);
    int ne = parent[v][e];
    if (ne < 0) {
      break;
    }
    v = g[v][e].to;
    e = ne;
  }
  cout << path.size() << ' ' << (ll) actAns << '\n';
  for (auto[L, x, R] : path) {
    cout << L + 1 << ' ' << x + 1 << ' ' << R + 1 << '\n';
  }
 
#ifdef GOLIKOV
  cerr << "Executed in " << chrono::duration_cast<chrono::milliseconds>(
      chrono::high_resolution_clock::now()
          - _clock_start).count() << "ms." << endl;
#endif
  return 0;
}