#pragma GCC target("sse,sse2,sse3,ssse3,sse4,popcnt,abm,mmx,tune=native")
#include <bits//stdc++.h>

#define ll long long
#define pb push_back
#define mp make_pair
#define all(x) x.begin(), x.end()
#define ld long double


const int mod = 1e9 + 9;
const int inf = 1e9 + 1;


using namespace std;


mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());


ll rnd(ll l, ll r) {
    uniform_int_distribution<ll> range(l, r);
    return range(rng);
}


template <class T>
istream& operator>>(istream &in, vector<T> &arr) {
    for (T &x : arr) {
        in >> x;
    }
    return in;
}


ll iterativePopcnt(ll val, int iter) {
    for (int i = 0; i < iter && val > 1; i++) { // TL if no "val > 1"
        val = __builtin_popcount(val);
    }
    return val;
}


struct solver {
    vector<ll> tMax, tMin, tPush, tSum;
    vector<int> tPopcnt;
    int n;

    solver(const vector<int32_t> &arr) {
        n = arr.size();
        tMax.resize(4 * n);
        tMin.resize(4 * n);
        tPush.resize(4 * n, -1);
        tSum.resize(4 * n);
        tPopcnt.resize(4 * n, 0);
        build(1, 0, n - 1, arr);
    }

    void recalc(int v) {
        tMax[v] = max(tMax[v * 2], tMax[v * 2 + 1]);
        tMin[v] = min(tMin[v * 2], tMin[v * 2 + 1]);
        tSum[v] = tSum[v * 2] + tSum[v * 2 + 1];
    }

    void push(int v, int len) {
        if (tPopcnt[v] > 0) {
            assert(tMin[v] == tMax[v]);
            ll val = tMin[v];
            ll newVal = iterativePopcnt(val, tPopcnt[v]);
            tSum[v] = newVal * (len + 1);
            tMin[v] = newVal;
            tMax[v] = newVal;
            if (len > 0) {
                if (tPush[v * 2] != -1) {
                    tPush[v * 2] = iterativePopcnt(tPush[v * 2], tPopcnt[v]);
                } else {
                    tPopcnt[v * 2] += tPopcnt[v];
                }
                if (tPush[v * 2 + 1] != -1) {
                    tPush[v * 2 + 1] = iterativePopcnt(tPush[v * 2 + 1], tPopcnt[v]);
                } else {
                    tPopcnt[v * 2 + 1] += tPopcnt[v];
                }
            }
            tPopcnt[v] = 0; // check forgot
        }
        if (tPush[v] == -1) {
            return;
        }
        ll x = tPush[v];
        tSum[v] = x * (len + 1);
        tMin[v] = x;
        tMax[v] = x;
        if (len > 0) {
            tPush[v * 2] = x;
            tPush[v * 2 + 1] = x;
            tPopcnt[v * 2] = 0; // Check forgot
            tPopcnt[v * 2 + 1] = 0;
        }
        tPush[v] = -1;
    }

    void build(int v, int l, int r, const vector<int> &arr) {
        if (l == r) {
            tMax[v] = arr[l];
            tMin[v] = arr[l];
            tSum[v] = arr[l];
            return;
        }
        int mid = (l + r) / 2;
        build(v * 2, l, mid, arr);
        build(v * 2 + 1, mid + 1, r, arr);
        recalc(v);
    }

    void modify(int v, int l, int r, int ql, int qr) {
        push(v, r - l);
        if (l > qr or r < ql or tMax[v] <= 1) { // check if 1 only or no this check
            return;
        }
        if (tMax[v] == tMin[v] && ql <= l && r <= qr) {
            tPopcnt[v]++;
            push(v, r - l); // check forgot
            return;
        }
        int mid = (l + r) / 2;
        modify(v * 2, l, mid, ql, qr);
        modify(v * 2 + 1, mid + 1, r, ql, qr);
        recalc(v);
    }

    void set(int v, int l, int r, int ql, int qr, int newVal) {
        push(v, r - l);
        if (l > qr || r < ql) {
            return;
        }
        if (ql <= l and r <= qr) {
            tPush[v] = newVal;
            push(v, r - l);
            return;
        }
        int mid = (l + r) / 2;
        set(v * 2, l, mid, ql, qr, newVal);
        set(v * 2 + 1, mid + 1, r, ql, qr, newVal);
        recalc(v);
    }

    ll sum(int v, int l, int r, int ql, int qr) {
        push(v, r - l);
        if (l > qr or r < ql) {
            return 0;
        }
        if (ql <= l and r <= qr) {
            return tSum[v];
        }
        int mid = (l + r) / 2;
        return sum(v * 2, l, mid, ql, qr) + sum(v * 2 + 1, mid + 1, r, ql, qr);
    }

    void set(int l, int r, int val) {
        set(1, 0, n - 1, l, r, val);
    }

    void modify(int ql, int qr) {
        modify(1, 0, n - 1, ql, qr);
    }

    ll sum(int ql, int qr) {
        return sum(1, 0, n - 1, ql, qr);
    }
};


void solve() {
    int n, m;
    cin >> n >> m;
    vector<int> arr(n);
    for (auto &x : arr) {
        cin >> x;
    }
    solver slv(arr);
    while (m--) {
        int type, l, r;
        cin >> type >> l >> r;
        if (type == 1) {
            l--;
            r--;
            cout << slv.sum(l, r) << "\n";
        } else if (type == 2) {
            int x;
            cin >> x;
            l--;
            r--;
            slv.set(l, r, x);
        } else {
            l--;
            r--;
            slv.modify(l, r);
        }
    }
}


int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    cout.precision(10);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    solve();
    return 0;
}